/*
 * Created on 2005. 5. 16.
 */
package net.javajigi.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Stack;
import java.util.StringTokenizer;

public class Configuration {
	private static class IncludeReader {

		public String readLine() throws IOException {
			String s;
			do {
				s = reader.readLine();
				if (s == null) {
					if (readerStack.size() == 0)
						break;
					reader = (BufferedReader) readerStack.pop();
					file = (File) fileStack.pop();
					continue;
				}
				s = s.trim();
				if (!s.startsWith("#!include"))
					break;
				String s1 = s.substring(9).trim();
				File file1 = getRelativeFile(file, s1);
				BufferedReader bufferedreader = getReader(file1);
				readerStack.push(reader);
				fileStack.push(file);
				reader = bufferedreader;
				file = file1;
			} while (true);
			return s;
		}

		private File getRelativeFile(File file1, String s) {
			if (!file1.isDirectory())
				file1 = file1.getParentFile();
			return new File(file1, s);
		}

		private BufferedReader getReader(File file1)
			throws FileNotFoundException {
			return new BufferedReader(new FileReader(file1));
		}

		private Stack readerStack;
		private Stack fileStack;
		private File file;
		private BufferedReader reader;

		public IncludeReader(File file1) throws FileNotFoundException {
			readerStack = new Stack();
			fileStack = new Stack();
			file = file1;
			reader = getReader(file1);
		}
	}

	private static class ConfigLineReader {

		public String readLine() throws IOException {
			String s = null;
			boolean flag = false;
			do {
				if (flag) {
					s = s + reader.readLine();
					flag = false;
				} else {
					s = reader.readLine();
				}
				if (s == null)
					break;
				s = s.trim();
				if (s.equals("") || s.startsWith("#") || s.startsWith("!"))
					continue;
				if (!s.endsWith("\\"))
					break;
				s = s.substring(0, s.length() - 1);
				flag = true;
			} while (true);
			return s;
		}

		private IncludeReader reader;

		public ConfigLineReader(IncludeReader includereader) {
			reader = includereader;
		}
	}

	private Map config;
	private static Configuration instance = new Configuration();	

	public Configuration(String s) throws FileNotFoundException, IOException {
		config = new HashMap();
		load(s);
	}

	public Configuration() {
		config = new HashMap();
	}

	private void load(String s) throws FileNotFoundException, IOException {
		ConfigLineReader configlinereader =
			new ConfigLineReader(new IncludeReader(new File(s)));
		do {
			String s1 = configlinereader.readLine();
			if (s1 != null)
				processLine(s1);
			else
				return;
		} while (true);
	}

	public List getPropertyNames() {
		return new ArrayList(config.keySet());
	}

	public static synchronized void init(String s)
		throws FileNotFoundException, IOException {
		instance.load(s);
	}

	public static void main(String args[]) {
		try {
			if (args.length < 1)
				System.out.println("Give me config file");
			Configuration configuration = new Configuration(args[0]);
			List list = configuration.getPropertyNames();
			for (Iterator iterator = list.iterator(); iterator.hasNext();) {
				String s = (String) iterator.next();
				String args1[] = configuration.getPropertyValueArray(s);
				for (int i = 0; i < args1.length; i++);
			}

		} catch (Exception exception) {
			exception.printStackTrace();
		}
	}

	private void processLine(String s) {
		int i = s.indexOf("=");
		if (i == -1) {
			throw new RuntimeException("= missing: " + s);
		} else {
			String s1 = s.substring(0, i).trim();
			String s2 = s.substring(i + 1).trim();
			String as[] = splitValues(s2);
			config.put(s1, as);
			return;
		}
	}

	private String processValue(String s) {
		StringBuffer stringbuffer = new StringBuffer();
		int i = 0;
		boolean flag = false;
		boolean flag1 = false;
		do {
			int j = s.indexOf("${", i);
			if (j == -1) {
				stringbuffer.append(s.substring(i));
				break;
			}
			int k = s.indexOf("}", j);
			if (k == -1) {
				stringbuffer.append(s.substring(i));
				break;
			}
			String s1 = s.substring(j + 2, k);
			stringbuffer.append(s.substring(i, j));
			stringbuffer.append(getPropertyValue(s1));
			i = k + 1;
		} while (true);
		return stringbuffer.toString();
	}

	private String[] splitValues(String s) {
		StringTokenizer stringtokenizer = new StringTokenizer(s, ";");
		ArrayList arraylist = new ArrayList();
		String s1;
		for (; stringtokenizer.hasMoreTokens(); arraylist.add(s1)) {
			s1 = stringtokenizer.nextToken();
			s1 = processValue(s1);
		}

		String as[] = new String[arraylist.size()];
		for (int i = 0; i < arraylist.size(); i++)
			as[i] = (String) arraylist.get(i);

		return as;
	}

	public String getPropertyValue(String s) {
		String as[] = getPropertyValueArray(s);
		if (as == null)
			return null;
		else
			return as[0];
	}

	public String[] getPropertyValueArray(String s) {
		return (String[]) config.get(s);
	}

	public static String get(String s) {
		return instance.getPropertyValue(s);
	}

	public static String[] getArray(String s) {
		return instance.getPropertyValueArray(s);
	}

	public static void set(String s, String s1) {
		instance.setPropertyValue(s, s1);
	}

	public static void setArray(String s, String as[]) {
		instance.setPropertyValueArray(s, as);
	}

	public void setPropertyValue(String s, String s1) {
		String as[] = new String[1];
		as[0] = s1;
		config.put(s, as);
	}

	public void setPropertyValueArray(String s, String as[]) {
		config.put(s, as);
	}

	public static void loadConfig(String s)
		throws FileNotFoundException, IOException {
		instance.load(s);
	}

	public static String getMessage(String s, Object aobj[]) {
		s = "msg." + s;
		String s1 = get(s);
		if (aobj == null)
			return s1;
		else
			return MessageFormat.format(s1, aobj);
	}

	public static String getMessage(String s) {
		return getMessage(s, null);
	}
}